import java.net.*;
import java.io.*;

/**
This class provides routines to look up look up the current value of a stock
or mutual fund over the net using a web site that provides such quotes.
Currently, it uses Yahoo's Finance site. It does this by sending a CGI query
with the String ticker symbol for the desired stock or fund in a way that the
CGI server can't distinguish it from a user typing the ticker symbol into the
appropriate text field on their web page. The resulting web page is parsed to
extract the desired info.

<p>
Note that since no instance variables are used in this class, the class is
thread-safe, meaning that it's public methods can be used by multiple threads
simultaneously without the possibility of corrupting any data.

<p>
Of course, this program will have to be changed if they change the format of
the result web page, change the request URL, etc.
*/
public class Quote {
  /**
  Server URL. You can use this URL in a browser to see the web page that
  allows people can type in ticker symbols into a text field.
  */
  static final String ServerURL = "http://quote.yahoo.com/";

  /**
  A proper URL for a basic query to this server for symbol sym would
  consist of this prefix string concat'ed with sym then with the suffix
  string below.
  */
  static final String URLQueryPrefix = ServerURL + "q?s=";
  static final String URLQuerySuffix = "&d=v1";

  /**
  The important information returned by the server is all in lines that
  start with the following prefix string.
  */
  static final String matchPrefix = "<td nowrap";

  /** The end-of-line indicator can be different on different platforms. */
  public static String newline = System.getProperty("line.separator");

  /**
  Utility method to strip the HTML tags out of a String. There are probably
  some valid types of HTML that could break this very dumb method, but it
  works fine for the lines returned by the quote server that we care about.
  */
  private static String removeHTMLTags(String s) {
    StringBuffer sb = new StringBuffer();
    boolean inTag = false;
    for (int i=0; i<s.length(); i++) {
      switch (s.charAt(i)) {
        case '<': inTag = true; break;
	case '>': inTag = false; break;
	default: if (!inTag) sb.append(s.charAt(i));
      }
    }
    return sb.toString();
  }

  /**
  Return the string form of the useful info about the given symbol.
  In the event of network IO problems, an error message String is returned.
  */
  static String getStringQuote(String symbol) throws MalformedURLException {
    StringBuffer sb = new StringBuffer();
    try {
      URL url = new URL(URLQueryPrefix + symbol + URLQuerySuffix);
      BufferedReader in =
        new BufferedReader(new InputStreamReader(url.openStream()));
      String line;
      int matchingLines = 0;
      while ((line = in.readLine()) != null) {
        // the important lines in the html code that Yahoo's server returns
        // are the first 6 of the 7 total that begin with matchPrefix
        if (line.startsWith(matchPrefix)) {
          if (++matchingLines < 7) {
	    if (matchingLines > 1) sb.append("\t");
            sb.append(removeHTMLTags(line));
	  }
	}
      }
    } catch (IOException e) {
      sb.append("IO problems. Try again later." + newline);
    }
    return sb.toString();
  }

  /*
  When this class's main method is run, it takes any number of ticker symbols
  as command-line arguments and prints to stdout quotes for each in turn in a
  nice table.
  */
  public static void main(String syms[]) throws MalformedURLException {
    if (syms.length < 1) {
      System.out.println("Usage: java Quote <tickersym1> " +
        "[<tickersym2>] [<tickersym3>] .....");
      System.exit(1);
    }

    // print table column headings
    System.out.println("Symbol\tLast Tr\tPrice\tChange\t%Change\tVolume");

    // might be nicer to process each symbol using a different thread
    for (int i=0; i<syms.length; i++) {
      System.out.println(getStringQuote(syms[i]));
    }
  }
}
